from configparser import ConfigParser
from typing import Dict, Any

import requests
from pkg_resources import resource_filename


class FreeGeoIpClient:
    """
    A class which enables consuming Freegeoip's RESTful API.

    Attributes
    ----------
    api_key : str
        provides API required to fetch data.
    api_endpoint : str
        stores API endpoint.

    Methods
    -------
    get_geo_location():
        Returns geo data for current location based on IP address that made the request.
    get_geo_location_for_ip_address(ip_address):
        Returns geo data for location based on provided IP address.
    """

    def __init__(self, api_key) -> None:
        """
        Constructs all the necessary attributes for the Client object.

        Parameters
        ----------
        api_key : str
            Freegeoip's API key.
        """
        self.api_key = api_key
        self.api_endpoint = self.get_api_endpoint

    @property
    def get_api_endpoint(self) -> str:
        """
        Returns
        -------
        API endpoint : str
            Freegeoip's API endpoint.
        """
        return self.__get_config_file()["API"]["ENDPOINT"]

    def __get_config_file(
        self,
        filename: str = resource_filename(__name__, "data/client.cfg"),
    ) -> ConfigParser:
        """
        Get contents of config file.

        Parameters
        ----------
        filename : str
            Config file to read.

        Returns
        -------
        config : ConfigParser
            contents of config file.
        """
        config = ConfigParser()
        config.read(filename)
        return config

    def get_geo_location(self) -> Dict[str, Any]:
        """
        Get geo data for current location based on IP address that made the request.

        Returns
        -------
        geo data : Dict[str, Any]
            geo data for current location.

        Raises
        ------
        JSONDecodeError, HTTPError, MissingSchema
        """
        return requests.get(f"{self.api_endpoint}/?apikey={self.api_key}").json()

    def get_geo_location_for_ip_address(
        self,
        ip_address: str,
    ) -> Dict[str, Any]:
        """
        Get geo data for location based on provided IP address.

        Parameters
        ----------
        ip_address : str
            Desired IP address.

        Returns
        -------
        geo data : Dict[str, Any]
            geo data for location based on provided IP address.

        Raises
        ------
        JSONDecodeError, HTTPError, MissingSchema
        """
        return requests.get(
            f"{self.api_endpoint}/{ip_address}?apikey={self.api_key}"
        ).json()


if __name__ == "__main__":
    pass



"""
freegeoip_client package.

Provides FreeGeoIpClient object which enables consuming
FreeGeoIp's RESTful API by providing your own API key.

Usage
-----
from freegeoip_client import FreeGeoIpClient

client = FreeGeoIpClient(api_key="some_api_key")

geo_data = client.get_geo_location()
geo_data_by_ip = client.get_geo_location_for_ip_address("8.8.8.8")
"""

from .client import FreeGeoIpClient  # noqa: F401



from setuptools import find_packages, setup


def get_long_description():
    with open("README.md") as file:
        return file.read()


setup(
    name="freegeoip-client",
    version="1.0.1",
    description="FreeGeoIp's RESTful API client for Python",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Kevin Furjan",
    author_email="kfurjan@gmail.com",
    url="https://github.com/kfurjan/python-freegeoip-client",
    project_urls={
        "GitHub Project": "https://github.com/kfurjan/python-freegeoip-client",
        "Issue Tracker": "https://github.com/kfurjan/python-freegeoip-client/issues",
    },
    packages=find_packages(
        include=["freegeoip_client", "freegeoip_client.*"],
    ),
    package_data={
        "freegeoip_client": ["data/*.cfg"],
    },
    install_requires=[
        "requests>=2.27.1",
    ],
    setup_requires=[
        "pytest-runner",
        "flake8",
    ],
    tests_require=[
        "pytest",
        "requests-mock",
    ],
    python_requires=">=3.6",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords=[
        "FreeGeoIp",
        "Client",
        "REST API Client",
    ],
    license="MIT",
)
